/*	
	BMCONF - (Smart) Boot Manager GNU/Linux Configurator
	(C)opyright Risko Gergely, 2000.
	This program can be licensed under the terms of GNU GPL v2 or above.
	
	PLEASE NOTE THIS PROGRAM IS ALPHA. IF IT HAS GOT A BUG, IT CAN
	PRODUCE A BAD PARTITION TABLE AND BOOT RECORD. USE AT YOUR OWN
	RISK!
*/

#include "includes.h"

enum
{
    DRIVE_OPTION=0,
    NAME_OPTION
};

enum
{
    PASS_RECORD_OPTION=0,
    PASS_PASSWORD_OPTION
};

enum
{
    SCHE_RECORD_OPTION=0,
    SCHE_TIME_OPTION
};


const char *add_opts[] =
{
    [DRIVE_OPTION]="drive",
    [NAME_OPTION]="name",
    NULL
};

const char *pass_opts[] =
{
    [PASS_RECORD_OPTION]="record",
    [PASS_PASSWORD_OPTION]="password",
    NULL
};

const char *sche_opts[] =
{
    [SCHE_RECORD_OPTION]="record",
    [SCHE_TIME_OPTION]="time",
    NULL
};

int main(int argc, char **argv)
{
    int c;
    int willbeinstalled=0;
    int listflag=0;
    char *drive=NULL;
    char *themename;
    unsigned char drivenum;	/* 128 is the first hd */
    unsigned char removeflag=0;	/* 0 - no removeete, 
				   1 - removeete */
    int removefloppy=0;	/* 0 - no floppy removeete
			   >0 - floppy id which will be removed */
    int removehard=0;	/* 0 - no harddisk removeete
			   >0 - harddisk id which will be removed */
    int removepartid=0;	/* the partition id if removehard>0 */
    int addfloppy=0;
    unsigned char addhard=0;
    unsigned char addpartid=0;
    char *passset;
    int passrec=0;
    char *scheset;
    int scherec=0;
    char *adddrive=NULL;
    char *addname=NULL;
    int long_index;
    
    static struct option long_options[] =
    {
#define VERSION_OPTION 0
	{"version", 0, 0, 0},
#define HELP_OPTION 1
	{"help", 0, 0, 0},
#define DEFRAG_OPTION 2
	{"defrag", 0, 0, 0},
#define DEFAULT_OPTION 3
	{"default", 1, 0, 0},
#define DELAY_OPTION 4
	{"delay", 1, 0, 0},
#define AUTOHIDE_OPTION 5
	{"autohide", 1, 0, 0},
#define AUTOACTIVE_OPTION 6
	{"autoactive", 1, 0, 0},
#define NOAUTOSCAN_OPTION 7
	{"noautoscan", 0, 0, 0},
#define LIST_OPTION 8
	{"list", 0, 0, 0},
#define REMOVE_OPTION 9
	{"remove", 1, 0, 0},
#define DEVICE_OPTION 10
	{"device", 1, 0, 0},
#define ADD_OPTION 11
	{"add", 1, 0, 0},
#define REMBYNUM_OPTION 12
	{"rembynum", 1, 0 ,0},
#define INSTALL_OPTION 13
	{"install", 1, 0, 0},
#define PASSWORD_OPTION 14
	{"password", 1, 0, 0},
#define SCHEDULE_OPTION 15
	{"schedule", 1, 0, 0},
#define SECLOCK_OPTION 16
	{"seclock", 0, 0, 0},
#define DIRECTBOOT_OPTION 17
	{"direct-boot", 1, 0, 0},
	{0,0,0,0}
    };
    
    opterr=0;
    while ((c=getopt_long(argc,argv,"",long_options,&long_index)) != -1)
    {
	char * subopts;
	
	switch (c)
	{
	    case 0:
		switch (long_index)
		{
		    char * atoi_test;
		    int atoiret;
		    		    
		    case HELP_OPTION:
			usage();
			break;
		    case LIST_OPTION:
			listflag=1;
			if (drive)
			    list(drive);
			else
		    	    usage();
			break;
		    case REMOVE_OPTION:
    			removeflag=1;
		        parsefdhdx(optarg, &removefloppy, &removehard, &removepartid);
			if (drive)
			{
			    removepart(drive,removefloppy,removehard,removepartid);
			}
			else
			{
			    usage();
			}
			break;
		    case ADD_OPTION:
			addname=NULL;
			adddrive=NULL;
			subopts=optarg;
			while (*subopts!='\0')
			{
			    char *value;
	    		    switch(getsubopt(&subopts, add_opts, &value))
			    {
				case DRIVE_OPTION:
				    if (value==NULL)
					usage();

				    parsefdhdxadd(value, &addfloppy, &addhard, &addpartid, &adddrive);
				    break;
				case NAME_OPTION:
				    if (value==NULL)
					usage();
				    asprintf(&addname,"%s",value);
				    break;
				default:
				    fprintf(stderr, "Unknown add option: %s\n", value);
				    usage();
			    }
			}
			if ((drive) && (addname) && ((addfloppy) || ((addhard)&&(addpartid))))
			    addpartition(drive, addfloppy, addhard, adddrive, addpartid, addname);
			else
		    	    usage();
			break;
		    case PASSWORD_OPTION:
			subopts=optarg;
			passset=NULL;
			passrec=0;
			while (*subopts!='\0')
			{
			    char *value;
	    		    switch(getsubopt(&subopts, pass_opts, &value))
			    {
				case PASS_RECORD_OPTION:
				    if (value==NULL)
					usage();
				    if (strcasecmp(value, "root")==0)
				    {
					passrec=-1;
				    }
				    else
				    {
					atoiret=verifiedatoi(value);
				        if (atoiret>0)
				        {
					    passrec=atoiret;
				        }
				        else
				        {
					    fprintf(stderr,"Bad option for password record\n");
					    exit(-4);
				        }
				    }

				    break;
				case PASS_PASSWORD_OPTION:
				    if (value==NULL)
					usage();
				    asprintf(&passset,"%s",value);
				    break;
				default:
				    fprintf(stderr, "Unknown password option: %s\n", value);
				    usage();
			    }
			}
			if ((passrec) && (passset))
			    password(drive, passrec-1, passset);
			else
		    	    usage();
			break;
		    case SCHEDULE_OPTION:
			subopts=optarg;
			scheset=NULL;
			scherec=0;
			while (*subopts!='\0')
			{
			    char *value;
	    		    switch(getsubopt(&subopts, sche_opts, &value))
			    {
				case SCHE_RECORD_OPTION:
				    if (value==NULL)
					usage();
				    atoiret=verifiedatoi(value);
				    if (atoiret>0)
				    {
					scherec=atoiret;
				    }
				    else
				    {
					fprintf(stderr,"Bad option for schedule record\n");
					exit(-4);
				    }

				    break;
				case SCHE_TIME_OPTION:
				    if (value==NULL)
					usage();
				    asprintf(&scheset,"%s",value);
				    break;
				default:
				    fprintf(stderr, "Unknown schedule option: %s\n", value);
				    usage();
			    }
			}
			if (scherec)
			    schedule(drive, scherec-1, scheset);
			else
			    usage();
			break;
		    case DEVICE_OPTION:
			driveparse(optarg, &drive, &drivenum);
		        if (willbeinstalled) install_main(themename, drive, drivenum);
			if (!drivecheck(drive))
			{
			    fprintf(stderr,"Unable to handle %s\n", drive);
			    exit(-2);
			}
			printf("Device is: %s. Device's bios number: %d\n", drive, drivenum);
			break;
		    case VERSION_OPTION:
		        fprintf(stderr,"BMCONF version %s\n", VERSION);
			exit(-1);
			break;
		    case DEFRAG_OPTION:
			if (drive) defrag(drive);
			else usage();
			break;
		    case SECLOCK_OPTION:
			if (drive) seclock(drive);
			else usage();
			break;
		    case DEFAULT_OPTION:
			if (drive) 
			{
			    atoiret=verifiedatoi(optarg);
			    if (atoiret>0)
			    {
				chdefault(drive, atoiret-1);
			    }
			    else
			    {
				fprintf(stderr,"Bad option for default\n");
				exit(-4);
			    }
			}
			break;
		    case DIRECTBOOT_OPTION:
			if (drive) 
			{
			    atoiret=verifiedatoi(optarg);
			    if (atoiret>0)
			    {
				directboot(drive, atoiret-1);
			    }
			    else
			    {
				fprintf(stderr,"Bad option for direct-boot\n");
				exit(-4);
			    }
			}
			break;
		    case AUTOHIDE_OPTION:
			if (drive) 
			{
			    atoiret=atoi(optarg);
			    asprintf(&atoi_test,"%d",atoiret);
			    if (strcmp(atoi_test, optarg)==0)
			    {
				autohide(drive, atoiret-1);
			        free(atoi_test);
			    }
			    else
			    {
				fprintf(stderr,"Bad option for default\n");
				free(atoi_test);
				exit(-4);
			    }
			}
			break;
		    case AUTOACTIVE_OPTION:
			if (drive) 
			{
			    atoiret=atoi(optarg);
			    asprintf(&atoi_test,"%d",atoiret);
			    if (strcmp(atoi_test, optarg)==0)
			    {
				autoactive(drive, atoiret-1);
			        free(atoi_test);
			    }
			    else
			    {
				fprintf(stderr,"Bad option for default\n");
				free(atoi_test);
				exit(-4);
			    }
			}
			break;
		    case REMBYNUM_OPTION:
			if (drive) 
			{
			    atoiret=atoi(optarg);
			    asprintf(&atoi_test,"%d",atoiret);
			    if (strcmp(atoi_test, optarg)==0)
			    {
				rembynum(drive, atoiret-1);
			        free(atoi_test);
			    }
			    else
			    {
				fprintf(stderr,"Bad option for default\n");
				free(atoi_test);
				exit(-4);
			    }
			}
			break;
		    case DELAY_OPTION:
			if (drive) 
			{
			    atoiret=atoi(optarg);
			    asprintf(&atoi_test,"%d",atoiret);
			    if (strcmp(atoi_test, optarg)==0)
			    {
				delay(drive, atoiret);
			        free(atoi_test);
			    }
			    else
			    {
				fprintf(stderr,"Bad option for default\n");
				free(atoi_test);
				exit(-4);
			    }
			}
			else usage();
			break;
		    case NOAUTOSCAN_OPTION:
			if (drive)
			{
			    noautoscan(drive);
			}
			else usage();
			break;
		    case INSTALL_OPTION:
			willbeinstalled=1;
			asprintf(&themename, "%s", optarg);
		}
		break;
	    case '?':
		usage();
		break;
	    default:
		fprintf(stderr,"error\n");
		abort();
	}
    }
    
    if (!drive)
	usage();
        
    return 0;
}
