#define BLOCK_SIZE	512		/* The block size. It is 512. */
#define BR_GOOD_FLAG    0XAA55          /* Good Boot Record Flag */
#define BR_FLAG_OFF     0x01FE          /* the Offset of Boot Record Flag */
#define PART_TBL_OFF    0x01BE          /* the offset of the partition table */

#define BOOT_OFF        0x7C00          /* boot sector startup offset (in mem)*/
#define PART_OFF        0x0600          /* partition table offset (in mem) */
#define SECTOR_SIZE     0x200           /* size of a sector */

#define KERNEL_OFF      0x8000          /* Smart Boot Manager kernel startup
                                           Offset ( Seg = 0 ) */

#define KERNEL_SEG      0x0800          /* Segment ( Off = 0 ) */

#define SBMK_MAGIC      0x4B4D4253      /* magic number of
                                           Smart Boot Manager kernel. */
#define SBMK_VERSION    0x0303          /* version of kernel. */
#define SBMT_MAGIC      0x544D4253      /* magic number of
                                           Smart Boot Manager theme. */
#define SBMT_VERSION    0x0303          /* version of theme. */
#define SBML_MAGIC      0x4C4D4253      /* magic number of
                                           Smart Boot Manager loader. */
#define SBML_VERSION    0x0300          /* version of loader. */

#define MAX_SBM_SIZE    24576           /* the max size of Smart Boot Manager */

#define MAX_RECORD_NUM  32

/* the macros that used in struct struc_sbmk_data */
#define KNLFLAG_FIRSTRUN    0x01        /* this flag indicates that the kernel
                                           runs the first time. */
#define KNLFLAG_SECURITY    0x02        /* this flag indicates that the kernel
                                           is in security lock mode */
#define KNLFLAG_FLOPPY_OK   0x04        /* this flag indicates that the floppy
                                           driver is ok */
#define KNLFLAG_NOAUTOSCAN  0x10

/* the macros that used in struct struc_bootrecord */

#define INFOFLAG_DRIVEOK    0x01
#define INFOFLAG_REMOVABLE  0x04

#define INFOFLAG_LOGICAL    0x10   
#define INFOFLAG_HIDDEN     0x20
#define INFOFLAG_ACTIVE     0x40
#define INFOFLAG_AUTOACTIVE 0x02
#define INFOFLAG_AUTOHIDE   0x08

#define EXTFLAG_SCHEDULED   0X01        /* the schedule data is available */
#define EXTFLAG_HAVEKEYS    0X02        /* the keystrokes is available */


/*=============================================================*\
 * the header structure of Smart Boot Manager Loader           *
 * the loader is stored in the first sector of the harddisk or *
 * floppy (MBR). see loader.asm for details.                   *
 *                                                             *
 * this data struct is stored from the first byte of the loader*
 * the kernel that mentioned below is the Smart BootManager's  *
 * main program, it has nothing to do with the linux kernel.   *
\*=============================================================*/

struct struc_sbml_header
{
      dword jmp_cmd;         /* cli and jmp command. */
      dword magic;           /* magic number. SBML_MAGIC */
      word  version;         /* loader version. SBML_VERSION */
      byte  kernel_sectors;  /* the total sectors that kernel
                                and theme used. */
      byte  reserved;        /* reserved */
      byte  kernel_copies;   /* the number of kernel copies stored in disk, 
                                must <= 3 */
      byte  kernel_drvid1;   /* the drive id of the first copy of kernel */
      dword kernel_lba1;     /* the LBA address of the first copy of kernel */
      byte  kernel_drvid2;   /* LBA address is the absolute sector address */
      dword kernel_lba2;
      byte  kernel_drvid3;
      dword kernel_lba3;
} __attribute__ ((packed));

/*=============================================================*\
 * the header structure of smart boot manager kernel           *
 * the kernel is stored in the sectors right after the loader  *
 * (in current install program), in fact, it can be stored into*
 * anywhere.                                                   *
 * this data struct is stored from the first byte of the kernel*
\*=============================================================*/

struct struc_sbmk_header
{
      dword jmp_cmd;         /* jmp and nop command. */
      dword magic;           /* magic number. SBMK_MAGIC */
      word  version;         /* kernel version. SBMK_VERSION */
      word  kernel_size;     /* the size of kernel code in bytes. */
      word  checksum_size;   /* the number of bytes that will
                                be counted in checksum. */
      byte  checksum;        /* checksum value of the kernel */
      byte  kernel_sectors;  /* the total sectors that kernel
                                and theme used. */
      byte  kernel_copies;   /* the number of kernel copies. */
      byte  kernel_drvid1;   /* the drive id of the first copy of kernel */
      dword kernel_lba1;     /* the LBA address of the first copy of kernel */
      byte  kernel_drvid2;   /* LBA address is the absolute sector address */
      dword kernel_lba2;
      byte  kernel_drvid3;
      dword kernel_lba3;
} __attribute__ ((packed));

/*=============================================================*\
 * the header structure of smart boot manager theme            *
 * the theme  is stored right after the kernel                 *
 * see sbminst.c for details.                                  *
 * this data struct is stored from the first byte of the kernel*
\*=============================================================*/

struct struc_sbmt_header
{
      dword magic;             /* magic number. SBMT_MAGIC */
      word  reserved;
      byte  lang[6];           /* language info. */
      word  version;           /* theme version. SBMT_VERSION */
      word  size;              /* theme size. */
} __attribute__ ((packed));

/*=============================================================*\
 * the structure of boot record, including removable drives and*
 * partitions.                                                 *
 * the boot records data is stored in struct struc_sbmk_data   *
 * see below.                                                  *
\*=============================================================*/

struct struc_bootrecord
{
/* 32 Bit basic boot record info */
      byte  drive_id;       /* drive id = 0 to 255, 0 = the first floppy
                               0x80 = the first harddisk */
                                 ; 
      byte  part_id;        /* partition id used in linux, 
                               1-4 for primary partitions,
                               > 5 for logical partitions. */

                            
      byte  flags;          /* type flags of this record, 
                               see macros INFOFLAG_x,
                               if INFOFLAG_REMOVABLE is set, this record
                               is a removable drive, otherwise it's a
                               partition on fixed drive */

      byte  type;           /* partition type,  0 = free, 0x83 = linux native */

      dword father_abs_addr;/* father partition's LBA address,
                               for primary partition and floppy, it is zero,
                               for logical partition, it's the LBA address of
                               the extended partition that this logical part
                               stayed in.

                               each logical partition have it's own father 
                               partition, the father of /dev/hda5 is the first
                               extended partition whose record is stored in
                               MBR's partition table. The record of /dev/hda6's
                               father partition is stored in the partition 
                               table of the first extended partition, it's the
                               second entry. */
                               

      dword abs_addr;       /* partition's abs LBA address */
      dword password;       /* password of this record, 0 = no passwd 
                               the password is encrypted by func calc_password
                               see file utils.asm */

      byte  name[16];       /* name of this record, zero ending. */

/* 32 Bit extended boot record info */
      byte ext_flags;       /* extended flags, see macros EXTFLAG_x */
      byte reserved;        /* reserved byte */
      word begin_time;      /* begin schedule time (in minutes) of a day 
                               = (begin hour) * 60 + (begin_minute) */
      word end_time;        /* end schedule time (in minutes) of a day */
                            
      word keystrokes[13];  /* keystrokes to be preloaded into the keyboard
                               buffer, while booting the OS. 
                               each key occupied 1 word, high byte is the 
                               keyboard scan code, low byte is the ascii code */

} __attribute__ ((packed));

/*=============================================================*\
 * the structure of Smart BootManager kernel's data area,      *
 * it is stored right after the struc_sbmk_header data         * 
\*=============================================================*/

struct struc_sbmk_data
{
      byte kernel_flags;    /* kernel flags, see macros KNLFLAGS_x 
                               the install program must set this field
                               to KNLFLAGS_FIRSTRUN | KNLFLAGS_FLOPPY_OK */

      byte delay_time;      /* delay time in second, 0 = boot the default
                               record directly, 255 = no delay, wait user
                               choose. */
      byte direct_boot_record; /* the bootrecord index in the boot_records 
                                  array.
                                  is this record is a valid bootrecord,
                                  Smart BootManager will boot this record
                                  directly and this field will be reset to
                                  0xff. So the value of this field only affect
                                  one time. */
      byte default_record;  /* the bootrecord index in the boot_records array,
                               this record will be booted automatically if
                               the delay time is up or ESC key is pressed. */
      dword root_password;  /* the adiministration password, encrypted by
                               func calc_password */

      struct struc_bootrecord boot_records[MAX_RECORD_NUM];
                            /* the array of boot records */                           
} __attribute__ ((packed));

/*

the data layout of Smart BootManager kernel (main.bin) is:

    Offset      Data type
====================================
     0          struc_sbmk_header  sbmk_header;
                struc_sbmk_data    sbmk_data;

     .
     .          program codes
     . 

kernel_size     struc_sbmt_header  sbmt_header;

     .
     .          theme data
     .


note: the kernel_size is a member of struct struc_sbmk_header

      the total size of the kernel and theme is:

      sbmk_header.kernel_size + sbmt_header.size

      the total sectors that kernel and theme occupied is :

      (sbmk_header.kernel_size + sbmt_header.size + 511) / 512

*/

