/*
 * sbminst.c
 *
 * the installation program for Smart BootManager
 * 
 * this program only supports Linux and DOS.
 *
 * Copyright (C) 2000, Suzhe, Lonius  See file COPYING for details.
 * 
 * Modified for configurator-installator. Now it only supports linux.
 * Copyright (C) 2000, Risko Gergely under the GPL. 
*/
#include "includes.h"
#include "bins/loader.h"
#include "bins/main.h"
#include "bins/theme-us.h"
#include "bins/theme-zh.h"
#include "install.h"

char *ldr_dev, *knl_dev;
unsigned char knl_devid;
struct stat st;

int knl_tsize;
FILE *bk_fp, *ui_fp;
byte buf_ldr[SECTOR_SIZE + 1], buf_knl[MAX_SBM_SIZE];
byte buf_theme[MAX_THEME_SIZE];
byte *theme_code;

int always_yes;

struct sbml_header *p_loader;
struct sbmk_header *p_knl;
struct sbmt_header *p_theme;

void
init_settings ()
{
  knl_tsize = 0;		/* kernel part total size */
  bk_fp = NULL;
  ui_fp = NULL;
  theme_code = theme_us_code;
  always_yes = 0;
}

void
die (char *fmt, ...)
{
  va_list ap;
  va_start (ap, fmt);
  vfprintf (stderr, fmt, ap);
  va_end (ap);
  fprintf (stderr, "\n");
  exit (-1);
}

void install_params (char * themename, char *drive, unsigned char drivenum)
{
  int theme_size;
  FILE *theme_fp;

  if (strcmp (themename, "zh") == 0)
    theme_code = theme_zh_code;
  else if (strcmp (themename, "us") == 0)
    theme_code = theme_us_code;
  else
    {
      if ((theme_fp = fopen (themename, "rb")) == NULL)
	die ("fopen %s: %s", themename, strerror (errno));
      fseek (theme_fp, 0, SEEK_END);
      theme_size = ftell (theme_fp);
      rewind (theme_fp);
      if (fread (buf_theme, (theme_size > 16384) ? 16384 : theme_size,
	  1, theme_fp) < 1)
	    die ("fread %s: %s", themename, strerror (errno));
      theme_code = buf_theme;
      fclose (theme_fp);
    }
  
  ldr_dev=drive;
  knl_dev=drive;
  knl_devid=drivenum;
}

int
open_blk_dev (char *dev)
{
  int devd;
  if ((devd = open (dev, O_RDWR)) < 0)
    die ("open %s: %s", dev, strerror (errno));
  if (fstat (devd, &st) < 0)
    die ("stat %s: %s", dev, strerror (errno));
  if (!S_ISBLK (st.st_mode))
    die ("%s is not a block device!\n", dev);

  return devd;
}

void
close_blk_dev (int *dev_d)
{
  sync ();
  if (close (*dev_d) < 0)
    die ("close %s : %s", knl_dev, strerror (errno));
  *dev_d = 0;
}

void
proc_knl ()
{
  int devd;
  
  devd = open_blk_dev (knl_dev);

  memcpy (buf_knl, main_code, p_knl->kernel_size);
  memcpy (buf_knl + p_knl->kernel_size, theme_code, p_theme->size);

  p_knl = (struct sbmk_header *) buf_knl;

  p_knl->total_sectors = (knl_tsize + 511) / SECTOR_SIZE;
  p_knl->kernel_copies = 1;
  p_knl->kernel_addr1 = KNL_LBA;

  p_knl->kernel_drvid1 = knl_devid;

  p_knl->checksum = 0;
  p_knl->checksum = calc_checksum (buf_knl, p_knl->checksum_size);


  /* always begin at safe place */
  lseek (devd, SECTOR_SIZE, SEEK_SET);
  if (write (devd, buf_knl, knl_tsize) != knl_tsize)
    die ("write %s: %s", knl_dev, strerror (errno));

  close_blk_dev (&devd);
}

void
proc_ldr ()
{
  int devd;

  devd = open_blk_dev (ldr_dev);

  memcpy (buf_ldr, loader_code, LDR_SIZE);

  p_loader->kernel_drvid1 = p_knl->kernel_drvid1;
  p_loader->kernel_copies = 1;
  p_loader->kernel_addr1 = KNL_LBA;
  p_loader->kernel_sectors = p_knl->total_sectors;

  lseek (devd, 0, SEEK_SET);
  if (write (devd, buf_ldr, LDR_SIZE) != LDR_SIZE)
    die ("write %s: %s", ldr_dev, strerror (errno));

  close_blk_dev (&devd);

}

void
install_main (char * themename, char *drive, unsigned char drivenum)
{
  init_settings ();
  install_params (themename, drive, drivenum);

  p_theme = (struct sbmt_header *) theme_code;
  p_knl = (struct sbmk_header *) main_code;
  p_loader = (struct sbml_header *) buf_ldr;

  if (p_theme->magic != SBMT_MAGIC)
    die ("Invalid theme file!");
  if (p_theme->version != p_knl->version)
    die ("Theme version mismatched!");
  knl_tsize = p_knl->kernel_size + p_theme->size;
  if (p_theme->size > MAX_THEME_SIZE)
    die ("Theme file is too big!");

  proc_knl ();
  proc_ldr ();

  printf ("Installation successful!\n");
}
