#include "includes.h"

int verifiedatoi(char * string)
{
    char *atoi_test;
    int atoiret;
    
    atoiret=atoi(string);
    asprintf(&atoi_test,"%d",atoiret);
    if (strcmp(atoi_test, string)==0)
    {
	return atoiret;
    }
    else
    {
	return -1;
    }
}

byte calc_checksum (byte * knl, int size)
{
  char checksum;
  int i;
  for (i = 0, checksum = 0; i < size; i++)
    checksum += (*knl++);
  checksum = -(checksum);
  return checksum;
}

void usage()
{
    fprintf(stderr,"Smart Boot Manager UNIX Configurator 0.2 (with SBM 3.3-2)\n");
    fprintf(stderr,"(C)opyright Risko Gergely, 2000.\n");
    fprintf(stderr,"Can be licensed under the terms of GPL v2 or above\n");
    fprintf(stderr,"Usage: bmconf [options]\n");
    fprintf(stderr,"For more please view the bmconf.8 man page.\n");
    exit(-1);
}

char * drvidtostring(unsigned char id)
{
    static char retval[4];
    retval[3]=0;
    
    if (id<128)
    {
	sprintf(retval,"fd%1d",id+1);
    }
    else
    {
	sprintf(retval,"hd%1d",id-127);
    }
    
    return retval;
}

char * partidtostring(unsigned char partid)
{
    static char retval[7];
    retval[6]=0;

    if (partid<5)
    {
	if (partid)
	    sprintf(retval,"Pri %2d", partid);
	else
	    sprintf(retval,"Floppy");
    }
    else
    {
	sprintf(retval,"Log %2d", partid);
    }
    return retval;
}

void parsefdhdx(char *optarg, int *removefloppy, int *removehard, int *removepartid)
{
    *removefloppy=0;
    *removehard=0;
    *removepartid=0;
    
    if (optarg[1]!='d')
    {
        usage();
    }
    if (optarg[0]=='f')
    {
        *removefloppy=0;
	if (isdigit(optarg[2]) && (optarg[3]==0))
	{
	    sscanf(optarg,"fd%1d", removefloppy);
	}
	else
	{
	    usage();
	}
	if (!*removefloppy) 
	{
	    usage();
	}
    }
    else if (optarg[0]=='h') 
    {
	char * partidstr;

	*removehard=0;
	if (isdigit(optarg[2]) && (optarg[3]==':'))
	{
	    sscanf(optarg,"hd%1d",removehard);
	}
	else
	{
	    usage();
	}
	if (!*removehard) usage();
	*removepartid=atoi(optarg+4);
	asprintf(&partidstr,"%d",*removepartid);
	if (strlen(partidstr)!=strlen(optarg+4))
	    usage();
	free(partidstr);
    }
    else 
    {
	usage();
    }
}

void parsefdhdxadd(char *optarg, int *removefloppy, unsigned char *removehard, unsigned char *removepartid, char **removedrive)
{
    *removefloppy=0;
    *removehard=0;
    *removepartid=0;
    *removedrive=NULL;
    
    if (strncmp(optarg,"/dev/",5)!=0)
    {
        usage();
    }
    if ((optarg[5]=='f') && (optarg[6]=='d'))
    {
        *removefloppy=0;
	if (isdigit(optarg[7]) && (optarg[8]==0))
	{
	    if (sscanf(optarg,"/dev/fd%1d", removefloppy)!=1) 
	    {
	        usage();
	    }
	    asprintf(removedrive, "/dev/fd%1d", *removefloppy);
	    *removefloppy+=1;
	}
	else
	{
	    usage();
	}
    }
    else
    {
	char *tmpdrive;
	
	driveparse(optarg, &tmpdrive, removehard);
	partparse(tmpdrive, removedrive, removepartid);
	*removehard-=127;
	free(tmpdrive);
    }
}

int drivecheck(char *drive)
{
    int retval=1;
    int fd;

    fd=open(drive,O_RDWR);
    if (fd==-1) retval=0;
    else
    {
	struct struc_sbml_header head;
	
	if (sizeof(head)!=read(fd, &head, sizeof(head))) 
	{
	    perror("read");
	    retval=0;
	}
	if (SBML_MAGIC!=head.magic) 
	{
	    fprintf(stderr,"Wrong magic in sbml_header\n");
	    retval=0;
	}
	if (SBML_VERSION!=head.version)
	{
	    fprintf(stderr,"Wrong version in sbml_header\n");
	    retval=0;
	}
    	close(fd);
    }

    return retval;
}

int reccheck(struct struc_bootrecord record)
{
        if ((record.flags & INFOFLAG_DRIVEOK) &&
	    ((record.flags & INFOFLAG_REMOVABLE) || (record.type!=0)))
	{
	    return 1;
	}
	else
	{
	    return 0;
	}
}

void driveparse(char *string, char **drive, unsigned char *drivenum)
{
    char *running;
    char *str2;
    char *token;
    
    str2=malloc(strlen(string)+1);
    strcpy(str2, string);
    str2[strlen(string)]=0;
    running=str2;
    asprintf(drive, "%s", strsep(&running, ":"));
    token=running;
    if (token)
    {
	if (sscanf(token, "%x", (unsigned int *) drivenum)!=1)
	{
	    usage();
	}
    }
    else
    {
    
	*drivenum=128;
    }
    free(str2);
}

void partparse(char *drive, char **device, unsigned char *partnum)
{
    int i;
    char *numstr;
        
    *device=NULL;
    *partnum=0;
    for (i=0;i<strlen(drive);i++)
    {
	if (isdigit(drive[i]))
	{
	    *partnum*=10;
	    *partnum+=drive[i]-48;
	}
	else
	{
	    *partnum=0;
	}
    }
    asprintf(&numstr, "%d", *partnum);
    *device=malloc(strlen(drive)-strlen(numstr)+1);
    (*device)[strlen(drive)-strlen(numstr)]=0;
    strncpy(*device, drive, strlen(drive)-strlen(numstr));    
    free(numstr);
}

